from setuptools import setup

setup(
    name='urlSigner',
    version='0.0.1',
    description='Library to sign a URL',
    license='MIT',
    packages=['urlSigner'],
    author='deanle17',
    author_email='lecungdinh@gmail.com',
    keywords=['url', 'signer'],
    url='https://github.com/deanle17/fsecure-assignment'
)
